@@ -44,17 +44,17 @@ class AlbumsView(BaseView):
 
     @log
     def __init__(self, window, player):
-        super().__init__('albums', _("Albums"), window, None)
+        super().__init__('albums', _("Albums"), window)
 
         self._queue = LifoQueue()
-        self._album_widget = AlbumWidget(player, self)
         self.player = player
+        self._album_widget = AlbumWidget(
+            player, self, self._header_bar, self._selection_toolbar)
         self.add(self._album_widget)
         self.albums_selected = []
         self.all_items = []
         self.items_selected = []
         self.items_selected_callback = None
-        self._add_list_renderers()
         self._init = True
 
     @log
@@ -66,12 +66,14 @@ class AlbumsView(BaseView):
 
     @log
     def _on_selection_mode_changed(self, widget, data=None):
+        super()._on_selection_mode_changed(widget, data)
+
         if (not self._header_bar.selection_mode
                 and grilo.changes_pending['Albums']):
             self._on_changes_pending()
 
     @log
-    def _setup_view(self, view_type):
+    def _setup_view(self):
         self._view = Gtk.FlowBox(
             homogeneous=True, hexpand=True, halign=Gtk.Align.FILL,
             valign=Gtk.Align.START, selection_mode=Gtk.SelectionMode.NONE,
@@ -94,16 +96,12 @@ class AlbumsView(BaseView):
 
     @log
     def _on_child_activated(self, widget, child, user_data=None):
-        item = child.media_item
-
-        # Toggle the selection when in selection mode
-        if self.selection_mode:
-            child.check.set_active(not child.check.get_active())
+        if self.props.selection_mode:
             return
 
+        item = child.media_item
         # Update and display the album widget if not in selection mode
-        self._album_widget.update(
-            item, self._header_bar, self._selection_toolbar)
+        self._album_widget.update(item)
 
         self._header_bar.props.state = HeaderBar.State.CHILD
         self._header_bar.props.title = utils.get_album_title(item)
@@ -197,10 +195,13 @@ class AlbumsView(BaseView):
 
     @log
     def _on_album_event_triggered(self, evbox, event, child):
-        if event.button is 3:
+        modifiers = Gtk.accelerator_get_default_mod_mask()
+        if ((event.state & modifiers) == Gdk.ModifierType.CONTROL_MASK
+                and not self.props.selection_mode):
             self._on_selection_mode_request()
-            if self.selection_mode:
-                child.check.set_active(True)
+
+        if self.props.selection_mode:
+            child.check.props.active = not child.check.props.active
 
     @log
     def _on_child_toggled(self, check, pspec, child):
@@ -211,7 +212,7 @@ class AlbumsView(BaseView):
                 and child.media_item in self.albums_selected):
             self.albums_selected.remove(child.media_item)
 
-        self.update_header_from_selection(len(self.albums_selected))
+        self._update_header_from_selection(len(self.albums_selected))
 
     @log
     def _get_selected_album_songs(self):
@@ -239,11 +240,11 @@ class AlbumsView(BaseView):
             GObject.signal_handler_block(child.check, child.check_handler_id)
 
             # Set the checkbutton state without emiting the signal
-            child.check.set_active(selected)
+            child.check.props.active = selected
 
             GObject.signal_handler_unblock(child.check, child.check_handler_id)
 
-        self.update_header_from_selection(len(self.albums_selected))
+        self._update_header_from_selection(len(self.albums_selected))
 
     @log
     def select_all(self):
