@@ -22,9 +22,7 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-import gi
-gi.require_version("Gd", "1.0")
-from gi.repository import Gd, GdkPixbuf, GObject, Gtk
+from gi.repository import GdkPixbuf, GObject, Gtk
 
 from gnomemusic import log
 from gnomemusic.grilo import grilo
@@ -42,13 +40,11 @@ class BaseView(Gtk.Stack):
         return '<BaseView>'
 
     @log
-    def __init__(self, name, title, window, view_type, use_sidebar=False,
-                 sidebar=None):
+    def __init__(self, name, title, window, use_sidebar=False, sidebar=None):
         """Initialize
         :param name: The view name
         :param title: The view title
         :param GtkWidget window: The main window
-        :param view_type: The Gtk view type
         :param use_sidebar: Whether to use sidebar
         :param sidebar: The sidebar object (Default: Gtk.Box)
         """
@@ -74,11 +70,11 @@ class BaseView(Gtk.Stack):
         self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
 
         # Setup the main view
-        self._setup_view(view_type)
+        self._setup_view()
 
         if use_sidebar:
             self.stack = Gtk.Stack(
-                transition_type=Gtk.StackTransitionType.SLIDE_RIGHT,)
+                transition_type=Gtk.StackTransitionType.SLIDE_RIGHT)
             dummy = Gtk.Frame(visible=False)
             self.stack.add_named(dummy, 'dummy')
             if sidebar:
@@ -94,10 +90,6 @@ class BaseView(Gtk.Stack):
         self._window = window
         self._header_bar = window.headerbar
         self._selection_toolbar = window.selection_toolbar
-        self._header_bar._select_button.connect(
-            'toggled', self._on_header_bar_toggled)
-        self._header_bar._cancel_button.connect(
-            'clicked', self._on_cancel_button_clicked)
 
         self.name = name
         self.title = title
@@ -108,8 +100,7 @@ class BaseView(Gtk.Stack):
 
         self._init = False
         grilo.connect('ready', self._on_grilo_ready)
-        self._header_bar.connect(
-            'notify::selection-mode', self._on_selection_mode_changed)
+        self.connect('notify::selection-mode', self._on_selection_mode_changed)
         grilo.connect('changes-pending', self._on_changes_pending)
 
         self.bind_property(
@@ -117,47 +108,17 @@ class BaseView(Gtk.Stack):
             GObject.BindingFlags.SYNC_CREATE)
 
         self.bind_property(
-            'selection-mode', self._header_bar, 'selection-mode')
+            'selection-mode', self._header_bar, 'selection-mode',
+            GObject.BindingFlags.BIDIRECTIONAL)
 
     @log
     def _on_changes_pending(self, data=None):
         pass
 
     @log
-    def _setup_view(self, view_type):
+    def _setup_view(self):
         """Instantiate and set up the view object"""
-        self._view = Gd.MainView(shadow_type=Gtk.ShadowType.NONE)
-        self._view.set_view_type(view_type)
-
-        self._view.click_handler = self._view.connect('item-activated',
-                                                      self._on_item_activated)
-        self._view.connect('selection-mode-request',
-                           self._on_selection_mode_request)
-
-        self._view.bind_property('selection-mode', self, 'selection_mode',
-                                 GObject.BindingFlags.BIDIRECTIONAL)
-
-        self._view.connect('view-selection-changed',
-                           self._on_view_selection_changed)
-
-        self._box.pack_start(self._view, True, True, 0)
-
-    @log
-    def _on_header_bar_toggled(self, button):
-        self.selection_mode = button.get_active()
-
-        if self.selection_mode:
-            self.set_player_visible(False)
-            select_toolbar = self._selection_toolbar
-            select_toolbar.add_to_playlist_button.set_sensitive(False)
-        else:
-            self.set_player_visible(self.player.current_song is not None)
-            self.unselect_all()
-
-    @log
-    def _on_cancel_button_clicked(self, button):
-        self.unselect_all()
-        self.props.selection_mode = False
+        pass
 
     @log
     def _on_grilo_ready(self, data=None):
@@ -174,17 +135,9 @@ class BaseView(Gtk.Stack):
             self._populate()
 
     @log
-    def _on_view_selection_changed(self, widget):
-        if not self.selection_mode:
-            return
-        items = self._view.get_selection()
-        self.update_header_from_selection(len(items))
-
-    @log
-    def update_header_from_selection(self, n_items):
+    def _update_header_from_selection(self, n_items):
         """Updates header during item selection."""
-        select_toolbar = self._selection_toolbar
-        select_toolbar.add_to_playlist_button.set_sensitive(n_items > 0)
+        self._selection_toolbar.props.items_selected = n_items
         self._header_bar.props.items_selected = n_items
 
     @log
@@ -193,7 +146,8 @@ class BaseView(Gtk.Stack):
 
     @log
     def _on_selection_mode_changed(self, widget, data=None):
-        pass
+        if not self.props.selection_mode:
+            self.unselect_all()
 
     @log
     def populate(self):
@@ -203,17 +157,13 @@ class BaseView(Gtk.Stack):
     def _retrieval_finished(self, klass):
         self.model[klass.iter][4] = klass.pixbuf
 
-    @log
-    def _add_list_renderers(self):
-        pass
-
     @log
     def _on_item_activated(self, widget, id, path):
         pass
 
     @log
     def _on_selection_mode_request(self, *args):
-        self._header_bar._select_button.clicked()
+        self.props.selection_mode = not self.props.selection_mode
 
     @log
     def get_selected_songs(self, callback):
@@ -237,18 +187,14 @@ class BaseView(Gtk.Stack):
         """Select all the available songs."""
         count = self._set_selection(True)
 
-        if count > 0:
-            select_toolbar = self._selection_toolbar
-            select_toolbar.add_to_playlist_button.set_sensitive(True)
-
-        self.update_header_from_selection(count)
+        self._selection_toolbar.props.items_selected = count
+        self._update_header_from_selection(count)
 
     @log
     def unselect_all(self):
         """Unselects all the selected songs."""
         self._set_selection(False)
-        select_toolbar = self._selection_toolbar
-        select_toolbar.add_to_playlist_button.set_sensitive(False)
+        self._selection_toolbar.props.items_selected = 0
         self._header_bar.props.items_selected = 0
 
     @log
