@@ -22,8 +22,9 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-from gettext import gettext as _, ngettext
-from gi.repository import Gio, GLib, GObject, Gtk, Gdk, Pango
+from gettext import gettext as _
+
+from gi.repository import Gdk, Gio, GLib, GObject, Gtk, Pango
 
 from gnomemusic import log
 from gnomemusic.grilo import grilo
@@ -31,6 +32,8 @@ from gnomemusic.player import DiscoveryStatus
 from gnomemusic.playlists import Playlists, StaticPlaylists
 from gnomemusic.views.baseview import BaseView
 from gnomemusic.widgets.notificationspopup import PlaylistNotification
+from gnomemusic.widgets.playlistcontextmenu import PlaylistContextMenu
+from gnomemusic.widgets.playlistcontrols import PlaylistControls
 from gnomemusic.widgets.playlistdialog import PlaylistDialog
 import gnomemusic.utils as utils
 
@@ -60,7 +63,7 @@ class PlaylistView(BaseView):
         sidebar_container.add(self._sidebar)
 
         super().__init__(
-            'playlists', _("Playlists"), window, None, True, sidebar_container)
+            'playlists', _("Playlists"), window, True, sidebar_container)
 
         self._window = window
         self.player = player
@@ -69,26 +72,10 @@ class PlaylistView(BaseView):
 
         self._add_list_renderers()
 
-        builder = Gtk.Builder()
-        builder.add_from_resource('/org/gnome/Music/PlaylistControls.ui')
-        headerbar = builder.get_object('grid')
-        self._name_stack = builder.get_object('stack')
-        self._name_label = builder.get_object('playlist_name')
-        self._rename_entry = builder.get_object('playlist_rename_entry')
-        self._rename_entry.connect('changed', self._on_rename_entry_changed)
-        self._rename_entry.connect(
-            'key-press-event', self._on_rename_entry_key_pressed)
-        self._rename_done_button = builder.get_object(
-            'playlist_rename_done_button')
-        self._songs_count_label = builder.get_object('songs_count')
-        self._menubutton = builder.get_object('playlist_menubutton')
+        self._pl_ctrls = PlaylistControls()
+        self._pl_ctrls.connect('playlist-renamed', self._on_playlist_renamed)
 
-        builder = Gtk.Builder()
-        builder.add_from_resource('/org/gnome/Music/PlaylistContextMenu.ui')
-        self._popover_menu = builder.get_object('song_menu')
-        self._song_popover = Gtk.Popover.new_from_model(
-            self._view, self._popover_menu)
-        self._song_popover.set_position(Gtk.PositionType.BOTTOM)
+        self._song_popover = PlaylistContextMenu(self._view)
 
         play_song = Gio.SimpleAction.new('play_song', None)
         play_song.connect('activate', self._play_song)
@@ -120,11 +107,10 @@ class PlaylistView(BaseView):
         self._window.add_action(self._playlist_rename_action)
 
         self._grid.insert_row(0)
-        self._grid.attach(headerbar, 1, 0, 1, 1)
+        self._grid.attach(self._pl_ctrls, 1, 0, 1, 1)
 
         sidebar_container.set_size_request(220, -1)
-        sidebar_container.get_style_context().add_class('side-panel')
-        self._sidebar.get_style_context().add_class('view')
+        sidebar_container.get_style_context().add_class('sidebar')
         self._sidebar.set_selection_mode(Gtk.SelectionMode.SINGLE)
         self._sidebar.connect('row-activated', self._on_playlist_activated)
 
@@ -139,10 +125,6 @@ class PlaylistView(BaseView):
         self.pls_todelete = {}
         self._songs_todelete = {}
         self._songs_count = 0
-        self._handler_rename_done_button = 0
-        self._handler_rename_entry = 0
-
-        self._update_songs_count()
 
         self.model.connect('row-inserted', self._on_song_inserted)
         self.model.connect('row-deleted', self._on_song_deleted)
@@ -156,11 +138,12 @@ class PlaylistView(BaseView):
         self.show_all()
 
     @log
-    def _on_changes_pending(self, data=None):
-        pass
+    def _update_songs_count(self, songs_count):
+        self._songs_count = songs_count
+        self._pl_ctrls.props.songs_count = songs_count
 
     @log
-    def _setup_view(self, view_type):
+    def _setup_view(self):
         view_container = Gtk.ScrolledWindow(hexpand=True, vexpand=True)
         self._box.pack_start(view_container, True, True, 0)
 
@@ -241,7 +224,7 @@ class PlaylistView(BaseView):
             cell.set_property('text', utils.get_album_title(item))
 
     def _on_list_widget_icon_render(self, col, cell, model, _iter, data):
-        if not self.player.running_playlist(
+        if not self.player.playing_playlist(
                 'Playlist', self._current_playlist.get_id()):
             cell.set_visible(False)
             return
@@ -262,7 +245,7 @@ class PlaylistView(BaseView):
     def _update_model(self, player, playlist, current_iter):
         if self._iter_to_clean:
             self._iter_to_clean_model[self._iter_to_clean][10] = False
-        if not player.running_playlist(
+        if not player.playing_playlist(
                 'Playlist', self._current_playlist.get_id()):
             return False
 
@@ -347,7 +330,7 @@ class PlaylistView(BaseView):
                 self.player.set_playlist(
                     'Playlist', self._current_playlist.get_id(), self.model,
                     _iter)
-                self.player.set_playing(True)
+                self.player.play()
 
         # 'row-activated' signal is emitted before 'drag-begin' signal.
         # Need to wait to check if drag and drop operation is active.
@@ -360,7 +343,7 @@ class PlaylistView(BaseView):
         :param Gtk.TreeView treeview: self._view
         :param Gdk.EventButton event: clicked event
         """
-        if event.button != 3:
+        if event.button != Gdk.BUTTON_SECONDARY:
             return
 
         path, col, cell_x, cell_y = treeview.get_path_at_pos(event.x, event.y)
@@ -373,7 +356,6 @@ class PlaylistView(BaseView):
         self._song_popover.set_relative_to(self._view)
         self._song_popover.set_pointing_to(rect)
         self._song_popover.popup()
-        return
 
     @log
     def _drag_begin(self, widget_, drag_context):
@@ -409,7 +391,7 @@ class PlaylistView(BaseView):
         last_pos = max(new_pos, prev_pos)
 
         # update player's playlist.
-        if self.player.running_playlist(
+        if self.player.playing_playlist(
                 'Playlist', self._current_playlist.get_id()):
             playing_old_path = self.player.current_song.get_path().to_string()
             playing_old_pos = int(playing_old_path)
@@ -464,7 +446,7 @@ class PlaylistView(BaseView):
             'song': song,
             'index': index
         }
-        self._remove_song_from_playlist(self._current_playlist, song)
+        self._remove_song_from_playlist(self._current_playlist, song, index)
         self._create_notification(PlaylistNotification.Type.SONG, song_id)
 
     @log
@@ -539,10 +521,10 @@ class PlaylistView(BaseView):
         playlist_name = utils.get_media_title(playlist)
 
         if self.rename_active:
-            self.disable_rename_playlist()
+            self._pl_ctrls.disable_rename_playlist()
 
         self._current_playlist = playlist
-        self._name_label.set_text(playlist_name)
+        self._pl_ctrls.props.playlist_name = playlist_name
         self._current_playlist_index = row.get_index()
 
         # if the active queue has been set by this playlist,
@@ -551,7 +533,7 @@ class PlaylistView(BaseView):
         self.model.clear()
         self._iter_to_clean = None
         self._iter_to_clean_model = None
-        self._songs_count = 0
+        self._update_songs_count(0)
         grilo.populate_playlist_songs(playlist, self._add_song)
 
         if self._current_playlist_is_protected():
@@ -588,7 +570,6 @@ class PlaylistView(BaseView):
         :param Gtk.ListStore model: model
         """
         if not song:
-            self._update_songs_count()
             self.emit('playlist-songs-loaded')
             return None
 
@@ -599,15 +580,9 @@ class PlaylistView(BaseView):
             index, [2, 3, 5, 9],
             [title, artist, song, song.get_favourite()])
 
-        self._songs_count += 1
+        self._update_songs_count(self._songs_count + 1)
         return iter_
 
-    @log
-    def _update_songs_count(self):
-        self._songs_count_label.set_text(
-            ngettext("%d Song", "%d Songs", self._songs_count)
-            % self._songs_count)
-
     @log
     def _on_play_activate(self, menuitem, data=None):
         _iter = self.model.get_iter_first()
@@ -622,18 +597,15 @@ class PlaylistView(BaseView):
     @log
     def _current_playlist_is_protected(self):
         current_playlist_id = self._current_playlist.get_id()
-        if current_playlist_id in StaticPlaylists().get_ids():
-            return True
-        else:
-            return False
+        return current_playlist_id in StaticPlaylists().get_ids()
 
     @log
     def _is_current_playlist(self, playlist):
         """Check if playlist is currently displayed"""
-        if (self._current_playlist
-                and playlist.get_id() == self._current_playlist.get_id()):
-            return True
-        return False
+        if self._current_playlist is None:
+            return False
+
+        return playlist.get_id() == self._current_playlist.get_id()
 
     @log
     def _get_removal_notification_message(self, type_, media_id):
@@ -683,7 +655,7 @@ class PlaylistView(BaseView):
                     or self._sidebar.get_row_at_index(index - 1))
         self._sidebar.remove(selection)
 
-        if self.player.running_playlist('Playlist', playlist_id):
+        if self.player.playing_playlist('Playlist', playlist_id):
             self.player.stop()
             self.set_player_visible(False)
 
@@ -713,11 +685,10 @@ class PlaylistView(BaseView):
                     and playlist.get_id() == self._current_playlist.get_id()):
                 iter_ = self._add_song_to_model(
                     song_todelete['song'], self.model, song_todelete['index'])
-                if self.player.running_playlist(
+                if self.player.playing_playlist(
                         'Playlist', self._current_playlist.get_id()):
                     path = self.model.get_path(iter_)
                     self.player.add_song(self.model, path, iter_)
-                self._update_songs_count()
             self._songs_todelete.pop(media_id)
 
     @log
@@ -737,50 +708,24 @@ class PlaylistView(BaseView):
             self._songs_todelete.pop(media_id)
 
     @GObject.Property(type=bool, default=False)
-    @log
     def rename_active(self):
         """Indicate if renaming dialog is active"""
-        return self._name_stack.get_visible_child_name() == 'renaming_dialog'
-
-    @log
-    def _on_rename_entry_changed(self, selection):
-        self._rename_done_button.set_sensitive(selection.get_text_length() > 0)
-
-    @log
-    def _on_rename_entry_key_pressed(self, widget, event):
-        if event.keyval == Gdk.KEY_Escape:
-            self.disable_rename_playlist()
-
-    @log
-    def disable_rename_playlist(self):
-        """disables rename button and entry"""
-        self._name_stack.set_visible_child(self._name_label)
-        self._rename_done_button.disconnect(self._handler_rename_done_button)
-        self._rename_entry.disconnect(self._handler_rename_entry)
+        return self._pl_ctrls.props.rename_active
 
     @log
     def _stage_playlist_for_renaming(self, menuitem, data=None):
         selection = self._sidebar.get_selected_row()
         pl_torename = selection.playlist
+        self._pl_ctrls.enable_rename_playlist(pl_torename)
 
-        def playlist_renamed_callback(widget):
-            new_name = self._rename_entry.get_text()
-            if not new_name:
-                return
+    @log
+    def _on_playlist_renamed(self, arguments, new_name):
+        selection = self._sidebar.get_selected_row()
+        selection.get_child().props.label = new_name
 
-            selection.get_child().set_text(new_name)
-            pl_torename.set_title(new_name)
-            playlists.rename(pl_torename, new_name)
-            self._name_label.set_text(new_name)
-            self.disable_rename_playlist()
-
-        self._name_stack.set_visible_child_name('renaming_dialog')
-        self._rename_entry.set_text(utils.get_media_title(pl_torename))
-        self._rename_entry.grab_focus()
-        self._handler_rename_entry = self._rename_entry.connect(
-            'activate', playlist_renamed_callback)
-        self._handler_rename_done_button = self._rename_done_button.connect(
-            'clicked', playlist_renamed_callback)
+        pl_torename = selection.playlist
+        pl_torename.set_title(new_name)
+        playlists.rename(pl_torename, new_name)
 
     @log
     def _on_playlist_created(self, playlists, playlist):
@@ -791,31 +736,25 @@ class PlaylistView(BaseView):
     def _on_song_added_to_playlist(self, playlists, playlist, item):
         if self._is_current_playlist(playlist):
             iter_ = self._add_song_to_model(item, self.model)
-            if self.player.running_playlist(
+            if self.player.playing_playlist(
                     'Playlist', self._current_playlist.get_id()):
                 path = self.model.get_path(iter_)
                 self.player.add_song(self.model, path, iter_)
 
     @log
-    def _remove_song_from_playlist(self, playlist, item):
-        if (self._is_current_playlist(playlist)):
-            model = self.model
-        else:
+    def _remove_song_from_playlist(self, playlist, item, index):
+        if not self._is_current_playlist(playlist):
             return
 
-        for row in model:
-            if row[5].get_id() == item.get_id():
-                iter_ = row.iter
-                if self.player.running_playlist(
-                        'Playlist', self._current_playlist.get_id()):
-                    path = model.get_path(iter_)
-                    self.player.remove_song(model, path)
-                model.remove(iter_)
-                break
+        model = self.model
 
-        self._songs_count -= 1
-        self._update_songs_count()
-        return
+        iter_ = model.get_iter_from_string(str(index))
+        if self.player.playing_playlist(
+                'Playlist', self._current_playlist.get_id()):
+            self.player.remove_song(model, model.get_path(iter_))
+        model.remove(iter_)
+
+        self._update_songs_count(self._songs_count - 1)
 
     @log
     def populate(self):
