@@ -27,14 +27,24 @@ from gi.repository import GObject, Gtk
 from gnomemusic import log
 from gnomemusic.albumartcache import Art
 from gnomemusic.grilo import grilo
-from gnomemusic.gstplayer import Playback
 from gnomemusic.widgets.coverstack import CoverStack
 from gnomemusic.widgets.disclistboxwidget import DiscBox
 import gnomemusic.utils as utils
 
 
+@Gtk.Template(resource_path='/org/gnome/Music/ArtistAlbumWidget.ui')
 class ArtistAlbumWidget(Gtk.Box):
 
+    __gtype_name__ = 'ArtistAlbumWidget'
+
+    _album_box = Gtk.Template.Child()
+    _cover = Gtk.Template.Child()
+    _disc_list_box = Gtk.Template.Child()
+    _title = Gtk.Template.Child()
+    _year = Gtk.Template.Child()
+
+    selection_mode = GObject.Property(type=bool, default=False)
+
     __gsignals__ = {
         'songs-loaded': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
@@ -43,9 +53,9 @@ class ArtistAlbumWidget(Gtk.Box):
         return '<ArtistAlbumWidget>'
 
     @log
-    def __init__(self, media, player, model, parent_view, header_bar,
-                 selection_mode_allowed, size_group=None,
-                 cover_size_group=None):
+    def __init__(
+            self, media, player, model, header_bar, selection_mode_allowed,
+            size_group=None, cover_size_group=None):
         super().__init__(orientation=Gtk.Orientation.HORIZONTAL)
 
         self._size_group = size_group
@@ -56,44 +66,39 @@ class ArtistAlbumWidget(Gtk.Box):
         self._artist = utils.get_artist_name(self._media)
         self._album_title = utils.get_album_title(self._media)
         self._model = model
-        self._parent_view = parent_view
         self._header_bar = header_bar
         self._selection_mode = False
         self._selection_mode_allowed = selection_mode_allowed
 
         self._songs = []
 
-        self._header_bar._select_button.connect(
-            'toggled', self._on_header_select_button_toggled)
-
-        ui = Gtk.Builder()
-        ui.add_from_resource('/org/gnome/Music/ArtistAlbumWidget.ui')
-
-        self.cover = ui.get_object('cover')
-
-        self.cover_stack = CoverStack(self.cover, Art.Size.MEDIUM)
+        self.cover_stack = CoverStack(self._cover, Art.Size.MEDIUM)
         self.cover_stack.update(self._media)
 
-        self._disc_listbox = ui.get_object('disclistbox')
-        self._disc_listbox.set_selection_mode_allowed(
-            self._selection_mode_allowed)
+        allowed = self._selection_mode_allowed
+        self._disc_list_box.props.selection_mode_allowed = allowed
 
-        ui.get_object('title').set_label(self._album_title)
+        self.bind_property(
+            'selection-mode', self._disc_list_box, 'selection-mode',
+            GObject.BindingFlags.BIDIRECTIONAL |
+            GObject.BindingFlags.SYNC_CREATE)
+
+        self._title.props.label = self._album_title
         year = utils.get_media_year(self._media)
+
         if year:
-            ui.get_object('year').set_label(year)
+            self._year.props.label = year
 
         if self._size_group:
-            self._size_group.add_widget(ui.get_object('box1'))
+            self._size_group.add_widget(self._album_box)
 
         if self._cover_size_group:
             self._cover_size_group.add_widget(self.cover_stack._stack)
 
-        self.pack_start(ui.get_object('ArtistAlbumWidget'), True, True, 0)
-
         grilo.populate_album_songs(self._media, self._add_item)
 
-    def create_disc_box(self, disc_nr, disc_songs):
+    @log
+    def _create_disc_box(self, disc_nr, disc_songs):
         disc_box = DiscBox(self._model)
         disc_box.set_songs(disc_songs)
         disc_box.set_disc_number(disc_nr)
@@ -102,36 +107,9 @@ class ArtistAlbumWidget(Gtk.Box):
         disc_box.props.show_favorites = False
         disc_box.props.show_song_numbers = True
         disc_box.connect('song-activated', self._song_activated)
-        disc_box.connect('selection-toggle', self._selection_mode_toggled)
 
         return disc_box
 
-    def _selection_mode_toggled(self, widget):
-        if not self._selection_mode_allowed:
-            return
-
-        self._selection_mode = not self._selection_mode
-        self._on_selection_mode_request()
-
-        return True
-
-    def _on_selection_mode_request(self):
-        self._header_bar._select_button.clicked()
-
-    def _on_header_select_button_toggled(self, button):
-        """Selection mode button clicked callback."""
-        if button.get_active():
-            self._selection_mode = True
-            self._disc_listbox.props.selection_mode = True
-            self._header_bar.props.selection_mode = True
-            self._parent_view.set_player_visible(False)
-        else:
-            self._selection_mode = False
-            self._disc_listbox.props.selection_mode = False
-            self._header_bar.props.selection_mode = False
-            if self._player.get_playback_status() != Playback.STOPPED:
-                self._parent_view.set_player_visible(True)
-
     @log
     def _add_item(self, source, prefs, song, remaining, data=None):
         if song:
@@ -147,8 +125,8 @@ class ArtistAlbumWidget(Gtk.Box):
                 discs[disc_nr].append(song)
 
         for disc_nr in discs:
-            disc = self.create_disc_box(disc_nr, discs[disc_nr])
-            self._disc_listbox.add(disc)
+            disc = self._create_disc_box(disc_nr, discs[disc_nr])
+            self._disc_list_box.add(disc)
             if len(discs) == 1:
                 disc.props.show_disc_label = False
 
@@ -157,20 +135,28 @@ class ArtistAlbumWidget(Gtk.Box):
 
     @log
     def _song_activated(self, widget, song_widget):
-        if self._selection_mode:
+        if self.props.selection_mode:
             return
 
         self._player.stop()
         self._player.set_playlist(
             'Artist', self._artist, song_widget.model, song_widget.itr)
-        self._player.set_playing(True)
+        self._player.play()
 
         return True
 
     @log
-    def set_selection_mode(self, selection_mode):
-        if self._selection_mode == selection_mode:
-            return
-        self._selection_mode = selection_mode
+    def select_all(self):
+        """Select all items"""
+        self._disc_list_box.select_all()
 
-        self._disc_listbox.props.selection_mode = selection_mode
+    @log
+    def select_none(self):
+        """Deselect all items"""
+        self._disc_list_box.select_none()
+
+    @log
+    def get_selected_songs(self):
+        """Return a list of selected songs."""
+        items = self._disc_list_box.get_selected_items()
+        return items
