@@ -47,14 +47,14 @@ class DiscSongsFlowBox(Gtk.FlowBox):
 
         :param int columns: The number of columns the widget uses
         """
-        super().__init__()
-        super().set_selection_mode(Gtk.SelectionMode.NONE)
+        super().__init__(selection_mode=Gtk.SelectionMode.NONE)
+
+        self._columns = 1
+        self.props.columns = columns
 
-        self._columns = columns
         self.get_style_context().add_class('discsongsflowbox')
 
     @GObject.Property(type=int, minimum=1, default=1)
-    @log
     def columns(self):
         """Number of columns for the song list
 
@@ -64,7 +64,6 @@ class DiscSongsFlowBox(Gtk.FlowBox):
         return self._columns
 
     @columns.setter
-    @log
     def columns(self, columns):
         """Set the number of columns to use
 
@@ -79,10 +78,11 @@ class DiscSongsFlowBox(Gtk.FlowBox):
         else:
             max_per_line = int(children_n / self._columns) + 1
 
-        self.set_max_children_per_line(max_per_line)
-        self.set_min_children_per_line(max_per_line)
+        self.props.max_children_per_line = max_per_line
+        self.props.min_children_per_line = max_per_line
 
 
+@Gtk.Template(resource_path='/org/gnome/Music/DiscBox.ui')
 class DiscBox(Gtk.Box):
     """A widget which compromises one disc
 
@@ -91,14 +91,17 @@ class DiscBox(Gtk.Box):
     """
     __gtype_name__ = 'DiscBox'
 
+    _disc_label = Gtk.Template.Child()
+    _disc_songs_flowbox = Gtk.Template.Child()
+
     __gsignals__ = {
         'selection-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
-        'selection-toggle': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'song-activated': (GObject.SignalFlags.RUN_FIRST, None, (Gtk.Widget,))
     }
 
     columns = GObject.Property(type=int, minimum=1, default=1)
     selection_mode = GObject.Property(type=bool, default=False)
+    selection_mode_allowed = GObject.Property(type=bool, default=True)
     show_disc_label = GObject.Property(type=bool, default=False)
     show_durations = GObject.Property(type=bool, default=False)
     show_favorites = GObject.Property(type=bool, default=False)
@@ -118,34 +121,25 @@ class DiscBox(Gtk.Box):
         self._model = model
         self._model.connect('row-changed', self._model_row_changed)
 
-        builder = Gtk.Builder()
-        builder.add_from_resource('/org/gnome/Music/ArtistAlbumWidget.ui')
-
-        self._label = builder.get_object('disclabel')
-        self._label.set_no_show_all(True)
-        self._disc_songs_flowbox = builder.get_object('discsongsflowbox')
-
         self.bind_property(
             'columns', self._disc_songs_flowbox, 'columns',
             GObject.BindingFlags.SYNC_CREATE)
         self.bind_property(
-            'show-disc-label', self._label, 'visible',
+            'show-disc-label', self._disc_label, 'visible',
             GObject.BindingFlags.SYNC_CREATE)
 
         self._selection_mode_allowed = True
         self._selected_items = []
         self._songs = []
 
-        self.pack_start(builder.get_object('disc'), True, True, 0)
-
     @log
     def set_disc_number(self, disc_number):
         """Set the dics number to display
 
         :param int disc_number: Disc number to display
         """
-        self._label.set_label(_("Disc {}").format(disc_number))
-        self._label.set_visible(True)
+        self._disc_label.props.label = _("Disc {}").format(disc_number)
+        self._disc_label.props.visible = True
 
     @log
     def set_songs(self, songs):
@@ -250,20 +244,18 @@ class DiscBox(Gtk.Box):
 
     @log
     def _song_activated(self, widget, event):
-        # FIXME: don't think keys work correctly, if they did ever
-        # even.
-        if (not event.button == Gdk.BUTTON_SECONDARY
-                or (event.button == Gdk.BUTTON_PRIMARY
-                    and event.state & Gdk.ModifierType.CONTROL_MASK)):
-            self.emit('song-activated', widget)
-            if self.props.selection_mode:
-                itr = widget.itr
-                self._model[itr][6] = not self._model[itr][6]
-        else:
+        modifiers = Gtk.accelerator_get_default_mod_mask()
+        if ((event.state & modifiers) == Gdk.ModifierType.CONTROL_MASK
+                and not self.selection_mode):
             self.emit('selection-toggle')
-            if self.props.selection_mode:
-                itr = widget.itr
-                self._model[itr][6] = True
+
+        if (event.button == Gdk.BUTTON_PRIMARY
+                and not self.props.selection_mode):
+            self.emit('song-activated', widget)
+
+        if self.props.selection_mode:
+            itr = widget.itr
+            self._model[itr][6] = not self._model[itr][6]
 
         return True
 
@@ -291,6 +283,8 @@ class DiscListBox(Gtk.Box):
         'selection-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
+    selection_mode_allowed = GObject.Property(type=bool, default=False)
+
     def __repr__(self):
         return '<DiscListBox>'
 
@@ -300,7 +294,6 @@ class DiscListBox(Gtk.Box):
         super().__init__(orientation=Gtk.Orientation.VERTICAL)
 
         self._selection_mode = False
-        self._selection_mode_allowed = False
         self._selected_items = []
 
     @log
@@ -311,6 +304,10 @@ class DiscListBox(Gtk.Box):
 
         self.bind_property(
             'selection-mode', widget, 'selection-mode',
+            GObject.BindingFlags.BIDIRECTIONAL |
+            GObject.BindingFlags.SYNC_CREATE)
+        self.bind_property(
+            'selection-mode-allowed', widget, 'selection-mode-allowed',
             GObject.BindingFlags.SYNC_CREATE)
 
     @log
@@ -350,7 +347,6 @@ class DiscListBox(Gtk.Box):
         self.foreach(child_select_none)
 
     @GObject.Property(type=bool, default=False)
-    @log
     def selection_mode(self):
         """selection mode getter
 
@@ -360,21 +356,12 @@ class DiscListBox(Gtk.Box):
         return self._selection_mode
 
     @selection_mode.setter
-    @log
     def selection_mode(self, value):
         """selection-mode setter
 
         :param bool value: Activate selection mode
         """
-        if not self._selection_mode_allowed:
+        if not self.props.selection_mode_allowed:
             return
 
         self._selection_mode = value
-
-    @log
-    def set_selection_mode_allowed(self, allowed):
-        """Set if selection mode is allowed
-
-        :param bool allowed: Allow selection mode
-        """
-        self._selection_mode_allowed = allowed
