@@ -46,14 +46,12 @@ class SelectionBarMenuButton(Gtk.MenuButton):
     def __repr__(self):
         return "<SelectionBarMenuButton>"
 
-    @log
     def __init__(self):
         super().__init__()
 
         self._items_selected = 0
 
     @GObject.Property(type=int, default=0, minimum=0)
-    @log
     def items_selected(self):
         """The number of items selected
 
@@ -63,7 +61,6 @@ class SelectionBarMenuButton(Gtk.MenuButton):
         return self._items_selected
 
     @items_selected.setter
-    @log
     def items_selected(self, value):
         """Set the number of items selected
 
@@ -98,6 +95,7 @@ class HeaderBar(Gtk.HeaderBar):
     _back_button = Gtk.Template.Child()
 
     items_selected = GObject.Property(type=int, default=0, minimum=0)
+    selection_mode_allowed = GObject.Property(type=bool, default=True)
     stack = GObject.Property(type=Gtk.Stack)
 
     def __repr__(self):
@@ -129,6 +127,9 @@ class HeaderBar(Gtk.HeaderBar):
         self.bind_property(
             "selection-mode", self._select_button, "visible",
             GObject.BindingFlags.INVERT_BOOLEAN)
+        self.bind_property(
+            "selection-mode", self._select_button, "active",
+            GObject.BindingFlags.BIDIRECTIONAL)
         self.bind_property(
             "stack", self._stack_switcher, "stack",
             GObject.BindingFlags.BIDIRECTIONAL |
@@ -136,8 +137,11 @@ class HeaderBar(Gtk.HeaderBar):
         self.bind_property(
             "items-selected", self._selection_menu, "items-selected")
 
+        self.connect(
+            "notify::selection-mode-allowed",
+            self._on_selection_mode_allowed_changed)
+
     @GObject.Property(type=bool, default=False)
-    @log
     def selection_mode(self):
         """Selection mode
 
@@ -147,7 +151,6 @@ class HeaderBar(Gtk.HeaderBar):
         return self._selection_mode
 
     @selection_mode.setter
-    @log
     def selection_mode(self, mode):
         """Set the selection mode
 
@@ -164,7 +167,6 @@ class HeaderBar(Gtk.HeaderBar):
         self._update()
 
     @GObject.Property
-    @log
     def state(self):
         """State of the widget
 
@@ -174,7 +176,6 @@ class HeaderBar(Gtk.HeaderBar):
         return self._state
 
     @state.setter
-    @log
     def state(self, value):
         """Set state of the of widget
 
@@ -218,6 +219,11 @@ class HeaderBar(Gtk.HeaderBar):
 
         self.searchbar.reveal(False)
 
+    @Gtk.Template.Callback()
+    @log
+    def _on_cancel_button_clicked(self, button):
+        self.props.selection_mode = False
+
     @log
     def _update(self):
         if self.props.selection_mode:
@@ -232,3 +238,10 @@ class HeaderBar(Gtk.HeaderBar):
             and self.props.state != HeaderBar.State.MAIN
             and self.props.state != HeaderBar.State.EMPTY
         )
+
+    @log
+    def _on_selection_mode_allowed_changed(self, widget, data):
+        if self.props.selection_mode_allowed:
+            self._select_button.props.sensitive = True
+        else:
+            self._select_button.props.sensitive = False
