@@ -28,7 +28,7 @@ from gi.repository import GObject, Gtk
 from gnomemusic import log
 from gnomemusic.albumartcache import Art
 from gnomemusic.gstplayer import Playback
-from gnomemusic.player import RepeatType
+from gnomemusic.player import RepeatMode
 from gnomemusic.widgets.coverstack import CoverStack
 from gnomemusic.widgets.smoothscale import SmoothScale  # noqa: F401
 import gnomemusic.utils as utils
@@ -64,9 +64,10 @@ class PlayerToolbar(Gtk.ActionBar):
         return '<PlayerToolbar>'
 
     @log
-    def __init__(self, player):
+    def __init__(self, player, headerbar):
         super().__init__()
 
+        self._headerbar = headerbar
         self._player = player
         self._progress_scale.player = self._player.get_gst_player()
 
@@ -75,11 +76,13 @@ class PlayerToolbar(Gtk.ActionBar):
 
         self._sync_repeat_image()
 
+        self._headerbar.connect(
+            'notify::selection-mode', self._on_selection_mode_changed)
         self._player.connect('clock-tick', self._on_clock_tick)
         self._player.connect('song-changed', self._update_view)
         self._player.connect('prev-next-invalidated', self._sync_prev_next)
         self._player.connect('repeat-mode-changed', self._sync_repeat_image)
-        self._player.connect('state-changed', self._sync_playing)
+        self._player.connect('playback-status-changed', self._sync_playing)
 
     @Gtk.Template.Callback()
     @log
@@ -114,23 +117,31 @@ class PlayerToolbar(Gtk.ActionBar):
     def _on_next_button_clicked(self, button):
         self._player.next()
 
+    @log
+    def _on_selection_mode_changed(self, headerbar, selection_mode):
+        if self._headerbar.props.selection_mode:
+            self.hide()
+        elif self._player.playing:
+            self.show()
+
     @log
     def _sync_repeat_image(self, player=None):
         icon = None
-        if self._player.repeat == RepeatType.NONE:
+        if self._player.repeat == RepeatMode.NONE:
             icon = 'media-playlist-consecutive-symbolic'
-        elif self._player.repeat == RepeatType.SHUFFLE:
+        elif self._player.repeat == RepeatMode.SHUFFLE:
             icon = 'media-playlist-shuffle-symbolic'
-        elif self._player.repeat == RepeatType.ALL:
+        elif self._player.repeat == RepeatMode.ALL:
             icon = 'media-playlist-repeat-symbolic'
-        elif self._player.repeat == RepeatType.SONG:
+        elif self._player.repeat == RepeatMode.SONG:
             icon = 'media-playlist-repeat-song-symbolic'
 
         self._repeat_image.set_from_icon_name(icon, Gtk.IconSize.MENU)
 
     @log
     def _sync_playing(self, player):
-        self.show()
+        if not self._headerbar.props.selection_mode:
+            self.show()
 
         if self._player.get_playback_status() == Playback.PLAYING:
             image = self._pause_image
