@@ -77,6 +77,9 @@ class SongWidget(Gtk.EventBox):
         PLAYING = 1
         UNPLAYED = 2
 
+    def __repr__(self):
+        return '<SongWidget>'
+
     @log
     def __init__(self, media):
         super().__init__()
@@ -153,7 +156,6 @@ class SongWidget(Gtk.EventBox):
         self._star_image.props.hover = False
 
     @GObject.Property(type=bool, default=False)
-    @log
     def selection_mode(self):
         """Selection mode
 
@@ -163,7 +165,6 @@ class SongWidget(Gtk.EventBox):
         return self._selection_mode
 
     @selection_mode.setter
-    @log
     def selection_mode(self, value):
         """Set the selection mode
 
@@ -176,7 +177,6 @@ class SongWidget(Gtk.EventBox):
             self.props.selected = False
 
     @GObject.Property
-    @log
     def state(self):
         """State of the widget
 
