@@ -33,7 +33,7 @@ from gi.repository import Gtk, Gdk, Gio, GLib
 from gettext import gettext as _
 
 from gnomemusic import log
-from gnomemusic.player import Player, RepeatType
+from gnomemusic.player import Player, RepeatMode
 from gnomemusic.query import Query
 from gnomemusic.utils import View
 from gnomemusic.views.albumsview import AlbumsView
@@ -75,7 +75,6 @@ class Window(Gtk.ApplicationWindow):
         self.add_action(select_none)
 
         self.set_size_request(200, 100)
-        self.set_default_icon_name('org.gnome.Music')
 
         self.prev_view = None
         self.curr_view = None
@@ -208,10 +207,10 @@ class Window(Gtk.ApplicationWindow):
     @log
     def _setup_view(self):
         self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        self.player = Player(self)
-        self.player_toolbar = PlayerToolbar(self.player)
-        self.selection_toolbar = SelectionToolbar()
         self.headerbar = HeaderBar()
+        self.player = Player(self)
+        self.player_toolbar = PlayerToolbar(self.player, self.headerbar)
+        self.selection_toolbar = SelectionToolbar()
         self.views = [None] * len(View)
         self._stack = Gtk.Stack(
             transition_type=Gtk.StackTransitionType.CROSSFADE,
@@ -256,8 +255,8 @@ class Window(Gtk.ApplicationWindow):
             'toggled', self._on_search_toggled)
         self.headerbar.connect(
             'notify::selection-mode', self._on_selection_mode_changed)
-        self.selection_toolbar.add_to_playlist_button.connect(
-            'clicked', self._on_add_to_playlist_button_clicked)
+        self.selection_toolbar.connect(
+            'add-to-playlist', self._on_add_to_playlist)
 
         self.headerbar.props.state = HeaderBar.State.MAIN
         self.headerbar.show()
@@ -368,17 +367,17 @@ class Window(Gtk.ApplicationWindow):
             # Toggle repeat on Ctrl + R
             if (event.keyval == Gdk.KEY_r
                     and modifiers == control_mask):
-                if self.player.get_repeat_mode() == RepeatType.SONG:
-                    self.player.set_repeat_mode(RepeatType.NONE)
+                if self.player.get_repeat_mode() == RepeatMode.SONG:
+                    self.player.set_repeat_mode(RepeatMode.NONE)
                 else:
-                    self.player.set_repeat_mode(RepeatType.SONG)
+                    self.player.set_repeat_mode(RepeatMode.SONG)
             # Toggle shuffle on Ctrl + S
             if (event.keyval == Gdk.KEY_s
                     and modifiers == control_mask):
-                if self.player.get_repeat_mode() == RepeatType.SHUFFLE:
-                    self.player.set_repeat_mode(RepeatType.NONE)
+                if self.player.get_repeat_mode() == RepeatMode.SHUFFLE:
+                    self.player.set_repeat_mode(RepeatMode.NONE)
                 else:
-                    self.player.set_repeat_mode(RepeatType.SHUFFLE)
+                    self.player.set_repeat_mode(RepeatMode.SHUFFLE)
             # Go back from Album view on Alt + Left
             if (event.keyval == Gdk.KEY_Left
                     and modifiers == mod1_mask):
@@ -413,12 +412,12 @@ class Window(Gtk.ApplicationWindow):
             if (event.keyval == Gdk.KEY_Delete
                     and child == self.views[View.PLAYLIST]):
                 self.views[View.PLAYLIST].remove_playlist()
-            # Close search bar after Esc is pressed
+            # Close selection mode or search bar after Esc is pressed
             if event.keyval == Gdk.KEY_Escape:
-                self.headerbar.searchbar.reveal(False)
-                # Also disable selection
                 if self.headerbar.props.selection_mode:
                     self.headerbar.props.selection_mode = False
+                else:
+                    self.headerbar.searchbar.reveal(False)
 
         # Open the search bar when typing printable chars.
         key_unic = Gdk.keyval_to_unicode(event.keyval)
@@ -474,8 +473,8 @@ class Window(Gtk.ApplicationWindow):
             self.views[View.EMPTY],
             self.views[View.PLAYLIST]
         ]
-        self.headerbar._select_button.set_sensitive(
-            self.curr_view not in no_selection_mode)
+        allowed = self.curr_view not in no_selection_mode
+        self.headerbar.props.selection_mode_allowed = allowed
 
         # Disable renaming playlist if it was active when leaving
         # Playlist view
@@ -511,14 +510,9 @@ class Window(Gtk.ApplicationWindow):
     def _on_selection_mode_changed(self, widget, data=None):
         if self.headerbar.props.selection_mode == False:
             self._on_changes_pending()
-        else:
-            child = self._stack.get_visible_child()
-            in_playlist = (child == self.views[View.PLAYLIST])
-            self.selection_toolbar.add_to_playlist_button.set_visible(
-                not in_playlist)
 
     @log
-    def _on_add_to_playlist_button_clicked(self, widget):
+    def _on_add_to_playlist(self, widget):
         if self._stack.get_visible_child() == self.views[View.PLAYLIST]:
             return
 
